"""
Groups

This module contains actions for viewing groups in PPA.

!!! danger "PPA Version"

    The actions in this module require PPA v2.10.x or newer.
"""
from typing import Optional, List, Union

from playbook_tools import tables

from ._api import get_client, create_records
from .records import Group


TABLE_HEADER = ["Name", "Users Synchronised", "Role Count", "User Count", "Task Count"]
TABLE_FIELDS = ["name", "user_sync", "roles_count", "members_count", "images_count"]


@create_records(Group)
def get_all(address: Optional[str] = None, api_key: Optional[str] = None) -> List[Group]:
    """Get all current groups from PPA.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Groups](../inputs_outputs/#group).

    ??? example
        ```YAML linenums="1"
        - ppa_api.groups.get_all:
          save: groups
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.groups()


@create_records(Group)
def get_by_name(name: str, address: Optional[str] = None, api_key: Optional[str] = None) -> Group:
    """Get the group with the supplied name.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __name:__ the group's name

    ??? success "Output"

        A single [Group](../inputs_outputs/#group).

    ??? example
        ```YAML linenums="1"
        - ppa_api.groups.get_by_name:
            name: example.group
          save: group
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.group_by_name(name)


def display(text: str, groups: Union[List[Group], Group]) -> None:
    """Display groups in a table.

    The table will have the following columns:

    - Display Name
    - Logon Name
    - Email Address
    - Can Start Tasks

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __groups:__ any number of [Groups](../inputs_outputs/#group) to display in the table

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - ppa_api.groups.get_all:
          save: groups

        - ppa_api.groups.display:
            text: All Groups
          load:
            groups: groups
        ```
    """
    tables.output_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=groups)


def select(
    text: str, groups: Union[List[Group], Group], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> tables.TableSelection:
    """Display groups in a table, & prompt the task operator to make a selection.

    The table will have the following columns:

    - Display Name
    - Logon Name
    - Email Address
    - Can Start Tasks

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __groups:__ any number of [Groups](../inputs_outputs/#group) to display in the table

        - __minimum:__ the minimum number of acceptable selections

        - __maximum:__ the maximum number of acceptable selections

    ??? success "Output"

        A [Selection](../../../tables/#selection) containing:

        - **total**: the number of selected [Groups](../inputs_outputs/#group)

        - **all**: a list of selected [Groups](../inputs_outputs/#group)

        - **first**: the first selected [Groups](../inputs_outputs/#group)

    ??? tip

        - If neither a _minimum_ or _maximum_ is provided, the task operator will be able to submit 0 selections.
        - If _minimum_ or _maximum_ are provided, the operation will repeat until the task operator makes a valid number
            of selections.

    ??? example

        ```YAML linenums="1"
        - ppa_api.groups.get_all:
          save: groups

        - ppa_api.groups.select:
            text: Select Groups
          load:
            groups: groups
          save: selection

        - ppa_api.groups.display:
            text: Selected Groups
          load:
            groups: selection.all
        ```
    """
    return tables.input_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=groups, minimum=minimum, maximum=maximum)


def select_one(text: str, groups: Union[List[Group], Group]) -> Group:
    """Display groups in a table, & prompt the task operator to select one.

    The table will have the following columns:

    - Display Name
    - Logon Name
    - Email Address
    - Can Start Tasks

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __groups:__ any number of [Groups](../inputs_outputs/#group) to display in the table

    ??? success "Output"

        A single [Group](../inputs_outputs/#group).

    ??? example

        ```YAML linenums="1"
        - ppa_api.groups.get_all:
          save: groups

        - ppa_api.groups.select:
            text: Select Groups
          load:
            groups: groups
          save: group

        - ppa_api.groups.display:
            text: Selected Group
          load:
            groups: group
        ```
    """
    return select(text, groups=groups, minimum=1, maximum=1)
