"""
Inventory

This module contains actions for viewing images in the PPA Inventory page.

!!! danger "PPA Version"

    The actions in this module require PPA v2.7.x or newer.
"""
from typing import Optional, List, Union

from playbook_tools import tables

from ._api import get_client, create_records
from .records import Image


TABLE_HEADER = ["Name", "Description", "Author", "Updated At", "Deployed"]
TABLE_FIELDS = ["name", "description", "author", "updated_at", "deployed"]


@create_records(Image)
def get_latest(address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get the _latest version_ of each inventory image visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Imagess](../inputs_outputs/#image).

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_latest:
          save: latest_images
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.images(latest=True)


@create_records(Image)
def get_by_id(image_id: Union[int, str], address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get the inventory image with the supplied ID.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __image_id:__ the ID of the image

    ??? success "Output"

        A single [Image](../inputs_outputs/#image).

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_by_id:
            iamge_id: 123
          save: image
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.image_by_id(image_id)


@create_records(Image)
def get_all(address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get _all versions_ of each inventory image visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Images](../inputs_outputs/#image).

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_all:
          save: all_images
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.images(latest=False)


@create_records(Image)
def get_deployed(address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get all _deployed_ inventory images visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Images](../inputs_outputs/#image).

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_deployed:
          save: deployed_images
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return list(filter(lambda image: image.deployed, ppa.images(latest=True)))


@create_records(Image)
def get_all_undeployed(address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get all _undeployed_ inventory images visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Images](../inputs_outputs/#image).

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_all_undeployed:
          save: all_undeployed_images
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return list(filter(lambda image: not image.deployed, ppa.images(latest=False)))


@create_records(Image)
def get_latest_undeployed(address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get the _latest version_ of each _undeployed_ inventory image visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Images](../inputs_outputs/#image).

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_latest_undeployed:
          save: latest_undeployed_images
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return list(filter(lambda image: not image.deployed, ppa.images(latest=True)))


@create_records(Image)
def get_latest_by_name(name: str, address: Optional[str] = None, api_key: Optional[str] = None) -> Image:
    """Get the _latest version_ of an image with the supplied name.

    The image must be visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        Either a single [Image](../inputs_outputs/#image) or `null`.

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_latest_by_name:
            name: Local Group Audit
          save: image
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.image_by_name(name)


@create_records(Image)
def get_all_by_name(name: str, address: Optional[str] = None, api_key: Optional[str] = None) -> List[Image]:
    """Get _all versions_ of an image with the supplied name.

    The image must be visible to the current user.

    !!! question "Minimum Plugin Version: 2.0.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        Either a list of [Images](../inputs_outputs/#image) or `null`.

    ??? example
        ```YAML linenums="1"
        - ppa_api.inventory.get_all_by_name:
            name: Local Group Audit
          save: images
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.images_by_name(name)


def display(text: str, images: Union[List[Image], Image]) -> None:
    """Display images in a table.

    The table will have the following columns:

    - Name
    - Description
    - Author
    - Updated At
    - Deployed

    !!! question "Minimum Plugin Version: 3.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __images:__ any number of [Images](../inputs_outputs/#image) to display in the table

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - ppa_api.images.get_deployed:
          save: deployed_images

        - ppa_api.images.display:
            text: Deployed Images
          load:
            images: deployed_images
        ```
    """
    tables.output_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=images)


def select(
    text: str, images: Union[List[Image], Image], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> tables.TableSelection:
    """Display images in a table, & prompt the task operator to make a selection.

    The table will have the following columns:

    - Name
    - Description
    - Author
    - Updated At
    - Deployed

    !!! question "Minimum Plugin Version: 3.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __images:__ any number of [Images](../inputs_outputs/#image) to display in the table

        - __minimum:__ the minimum number of acceptable selections

        - __maximum:__ the maximum number of acceptable selections

    ??? success "Output"

        A [Selection](../../../tables/#selection) containing:

        - **total**: the number of selected [Images](../inputs_outputs/#image)

        - **all**: a list of selected [Images](../inputs_outputs/#image)

        - **first**: the first selected [Image](../inputs_outputs/#image)

    ??? tip

        - If neither a _minimum_ or _maximum_ is provided, the task operator will be able to submit 0 selections.
        - If _minimum_ or _maximum_ are provided, the operation will repeat until the task operator makes a valid number
            of selections.

    ??? example

        ```YAML linenums="1"
        - ppa_api.images.get_deployed:
          save: deployed_images

        - ppa_api.images.select:
            text: Select Images
          load:
            images: deployed_images
          save: selection

        - ppa_api.images.display:
            text: Selected Images
          load:
            images: selection.all
        ```
    """
    return tables.input_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=images, minimum=minimum, maximum=maximum)


def select_one(text: str, images: Union[List[Image], Image]) -> Image:
    """Display images in a table, & prompt the task operator to select one.

    The table will have the following columns:

    - Name
    - Description
    - Author
    - Updated At
    - Deployed

    !!! question "Minimum Plugin Version: 3.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __images:__ any number of [Images](../inputs_outputs/#image) to display in the table

    ??? success "Output"

        A single [Images](../inputs_outputs/#image).

    ??? example

        ```YAML linenums="1"
        - ppa_api.images.get_deployed:
          save: deployed_images

        - ppa_api.images.select_one:
            text: Select Image
          load:
            images: deployed_images
          save: selection
        ```
    """
    return select(text, images=images, minimum=1, maximum=1)
