"""
Users

This module contains actions for viewing users in PPA.

!!! danger "PPA Version"

    The actions in this module require PPA v2.7.x or newer.
"""
import csv
import logging

from io import StringIO
from typing import Optional, List, Union

from playbook_tools import tables

from ._api import get_client, create_records
from .records import User


logger = logging.getLogger(__name__)


TABLE_HEADER = ["Display Name", "Logon Name", "Email Address", "Can Start Tasks"]
TABLE_FIELDS = ["display_name", "username", "email", "can_start_tasks"]


@create_records(User)
def get_all(address: Optional[str] = None, api_key: Optional[str] = None) -> List[User]:
    """Get all current users from PPA.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Users](../inputs_outputs/#user).

    ??? example
        ```YAML linenums="1"
        - ppa_api.users.get_all:
          save: users
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.users()


@create_records(User)
def get_deleted(address: Optional[str] = None, api_key: Optional[str] = None) -> List[User]:
    """Get all deleted users from PPA.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        A list of [Users](../inputs_outputs/#user).

    ??? example
        ```YAML linenums="1"
        - ppa_api.users.get_deleted:
          save: deleted_users
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.deleted_users()


@create_records(User)
def get_by_logon_name(name: str, address: Optional[str] = None, api_key: Optional[str] = None) -> User:
    """Get the user with the supplied logon name.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __name:__ the user's logon name

    ??? success "Output"

        A single [User](../inputs_outputs/#user).

    ??? example
        ```YAML linenums="1"
        - ppa_api.users.get_by_logon_name:
            name: example.user
          save: user
        ```"""
    with get_client(address=address, api_key=api_key) as ppa:
        return ppa.user_by_username(name)


def access_report(address: Optional[str] = None, api_key: Optional[str] = None) -> str:
    """Generates a CSV export of the Users page in PPA.

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        This action takes no inputs.

    ??? success "Output"

        The generated CSV report as a string.

        See the snippet below for an example of how to download the CSV report from the Task Interface.

    ??? example
        ```YAML linenums="1"
        - ppa_api.users.access_report:
          save: report

        - ppa.ui.output_file:
            text: User Access Report
            filename: access_report.csv
          load:
            data: report
        ```
    """
    groups_to_roles = {}

    with get_client(address=address, api_key=api_key) as ppa:
        users = get_all()
        roles = [r for r in ppa.roles() if r.groups_count not in {None, 0}]

    buffer = StringIO()
    writer = csv.writer(buffer)
    writer.writerow(["Username", "Name", "Email", "Authentication Type", "Last Login", "Can Start Tasks", "Roles", "Permissions"])

    for user in users:
        user_roles = []
        for group in user.get("groups", []):
            if group not in groups_to_roles:
                # Find all roles containing this group & store it in the cache dictionary.
                groups_to_roles[group] = [r.name for r in roles if group in r.groups]
            user_roles.extend(groups_to_roles[group])
        writer.writerow(
            [
                user["username"],
                user["display_name"],
                user["email"] or "",
                "Active Directory" if user["username"].lower() != "admin" else "Local",
                user["last_logon"] or "",
                str(user["can_start_tasks"]),
                f"{', '.join(user_roles)}",
                f"{', '.join(user['permissions'])}" if user["permissions"] else "",
            ]
        )
    return buffer.getvalue()


def display(text: str, users: Union[List[User], User]) -> None:
    """Display users in a table.

    The table will have the following columns:

    - Display Name
    - Logon Name
    - Email Address
    - Can Start Tasks

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __users:__ any number of [Users](../inputs_outputs/#user) to display in the table

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - ppa_api.users.get_all:
          save: users

        - ppa_api.users.display:
            text: All Users
          load:
            users: users
        ```
    """
    tables.output_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=users)


def select(
    text: str, users: Union[List[User], User], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> tables.TableSelection:
    """Display users in a table, & prompt the task operator to make a selection.

    The table will have the following columns:

    - Display Name
    - Logon Name
    - Email Address
    - Can Start Tasks

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __users:__ any number of [Users](../inputs_outputs/#user) to display in the table

        - __minimum:__ the minimum number of acceptable selections

        - __maximum:__ the maximum number of acceptable selections

    ??? success "Output"

        A [Selection](../../../tables/#selection) containing:

        - **total**: the number of selected [Users](../inputs_outputs/#user)

        - **all**: a list of selected [Users](../inputs_outputs/#user)

        - **first**: the first selected [Users](../inputs_outputs/#user)

    ??? tip

        - If neither a _minimum_ or _maximum_ is provided, the task operator will be able to submit 0 selections.
        - If _minimum_ or _maximum_ are provided, the operation will repeat until the task operator makes a valid number
            of selections.

    ??? example

        ```YAML linenums="1"
        - ppa_api.users.get_all:
          save: users

        - ppa_api.users.select:
            text: Select Users
          load:
            users: users
          save: selection

        - ppa_api.users.display:
            text: Selected Users
          load:
            users: selection.all
        ```
    """
    return tables.input_table(text, header=TABLE_HEADER, extract=TABLE_FIELDS, records=users, minimum=minimum, maximum=maximum)


def select_one(text: str, users: Union[List[User], User]) -> User:
    """Display users in a table, & prompt the task operator to select one.

    The table will have the following columns:

    - Display Name
    - Logon Name
    - Email Address
    - Can Start Tasks

    !!! question "Minimum Plugin Version: 4.1.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __users:__ any number of [Users](../inputs_outputs/#user) to display in the table

    ??? success "Output"

        A single [User](../inputs_outputs/#user).

    ??? example

        ```YAML linenums="1"
        - ppa_api.users.get_all:
          save: users

        - ppa_api.users.select:
            text: Select Users
          load:
            users: users
          save: user

        - ppa_api.users.display:
            text: Selected User
          load:
            users: user
        ```
    """
    return select(text, users=users, minimum=1, maximum=1)
